<?php

namespace App\Http\Controllers\Admin;

use App\Http\Controllers\Controller;
use App\Models\Game;
use App\Models\User;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Validator;
use Illuminate\Validation\Rule;
use Spatie\Permission\Models\Permission;
use Spatie\Permission\Models\Role;

class Games extends Controller
{
    /**
     * Display a listing of the resource.
     */
    public function index()
    {
        //
        return view('admin.games.index', ['games' => Game::all()]);
    }

    /**
     * Show the form for creating a new resource.
     */
    public function create()
    {
        //
        return view('admin.games.create', ['users' => User::all()]);
    }

    /**
     * Store a newly created resource in storage.
     */
    public function store(Request $request)
    {
        //
        $validator = Validator::make($request->only(['name', 'short', 'manager']), [
            'name' => ['required', 'string', 'max:100', Rule::unique(Game::class)],
            'short' => ['required', 'string', 'max:20', Rule::unique(Game::class)],
            'manager' => ['required', Rule::exists('users', 'id')]
        ]);

        if($validator->fails()) {
            return redirect()->back()->withInput()->withErrors($validator->errors());
        }

        $input = $validator->validated();

        $game = new Game($request->only(['name', 'short']));
        $game->manager_id = $input['manager'];

        if(!$game->save()) {
            return redirect()->back()->withInput()->with('error', 'Game konnte nicht erstellt werden!');
        }

        $role = Role::findOrCreate($game->name);
        $perm = Permission::firstOrCreate(['name' => 'games.manage.'.$game->id], ['description' => 'Manage '.$game->name.' Teams', 'only_user' => true]);
        $game->manager->givePermissionTo($perm);
        $game->manager->assignRole($role);

        return redirect()->route('admin.games.index')->with('success', 'Game wurde erstellt.');
    }

    /**
     * Display the specified resource.
     */
    public function show(Game $game)
    {
        //
    }

    /**
     * Show the form for editing the specified resource.
     */
    public function edit(Game $game)
    {
        //
        return view('admin.games.edit', ['users' => User::all(), 'game' => $game]);
    }

    /**
     * Update the specified resource in storage.
     */
    public function update(Request $request, Game $game)
    {
        //
        $validator = Validator::make($request->only(['name', 'short', 'manager']), [
            'name' => ['required', 'string', 'max:100', Rule::unique(Game::class)->ignore($game->id)],
            'short' => ['required', 'string', 'max:20', Rule::unique(Game::class)->ignore($game->id)],
            'manager' => ['required', Rule::exists('users', 'id')]
        ]);

        if($validator->fails()) {
            return redirect()->back()->withInput()->withErrors($validator->errors());
        }

        $input = $validator->validated();
        $role = Role::findOrCreate($game->name);
        $perm = Permission::firstOrCreate(['name' => 'games.manage.'.$game->id], ['description' => 'Manage '.$game->name.' Teams', 'only_user' => true]);
        $manager = User::find($input['manager']);

        $game->manager->removeRole($role);
        $game->manager->revokePermissionTo($perm);

        $game->fill($request->only(['name', 'short']));
        $game->manager_id = $input['manager'];

        if(!$game->save()) {
            return redirect()->back()->withInput()->with('error', 'Game konnte nicht gespeichert werden!');
        }

        $manager->givePermissionTo($perm);
        $manager->assignRole($role);

        return redirect()->route('admin.games.index')->with('success', 'Game wurde gespeichert.');
    }

    public function delete(Game $game) {
        return view('admin.games.delete', ['game' => $game]);
    }

    /**
     * Remove the specified resource from storage.
     */
    public function destroy(Game $game)
    {
        if(!Role::where('name', $game->name)->delete() || !Permission::where('name', 'games.manage.'.$game->id)->delete() || !$game->delete()) {
            return redirect()->route('admin.games.index')->with('error', 'Game konnte nicht gelöscht werden.');
        }

        return redirect()->route('admin.games.index')->with('success', 'Game wurde gelöscht.');
    }
}
