<?php

namespace App\Http\Controllers;

use App\Models\Team;
use App\Models\User;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Validator;
use Illuminate\Validation\Rule;
use Spatie\Permission\Models\Permission;

class GameManageController extends Controller
{
    /**
     * Display a listing of the resource.
     */
    public function index()
    {
        //
        return view('gamemanage.index', ['teams' => Team::whereIn('game_id', auth()->user()->games()->pluck('id'))->get()]);
    }

    /**
     * Show the form for creating a new resource.
     */
    public function create()
    {
        if(!auth()->user()->can('teams.create')) {
            return redirect()->back()->with('error', 'Du hast nicht die benötigte Berechtigung!');
        }
        //
        return view('gamemanage.create', ['users' => User::all(), 'games' => auth()->user()->games]);
    }

    /**
     * Store a newly created resource in storage.
     */
    public function store(Request $request)
    {
        if(!auth()->user()->can('teams.create')) {
            return redirect()->back()->with('error', 'Du hast nicht die benötigte Berechtigung!');
        }
        //
        $validator = Validator::make($request->only(['name', 'short', 'leader', 'game']), [
            'name' => ['required', 'string', 'max:100', Rule::unique(Team::class)],
            'short' => ['required', 'string', 'max:20', Rule::unique(Team::class)],
            'leader' => ['required', Rule::exists('users', 'id')],
            'game' => ['required', Rule::exists('games', 'id')],
        ]);

        if($validator->fails()) {
            return redirect()->back()->withInput()->withErrors($validator->errors());
        }

        $input = $validator->validated();

        $team = new Team($request->only(['name', 'short']));
        $team->leader_id = $input['leader'];
        $team->game_id = $input['game'];

        if(!$team->save()) {
            return redirect()->back()->withInput()->with('error', 'Team konnte nicht erstellt werden!');
        }

        $perm = Permission::firstOrCreate(['name' => 'teams.manage.'.$team->id], ['description' => 'Manage Team '.$team->name, 'only_user' => true]);
        $team->leader->givePermissionTo($perm);

        return redirect()->route('gamemanage.index')->with('success', 'Team wurde erstellt.');
    }

    /**
     * Display the specified resource.
     */
    public function show(Team $team)
    {
        //
    }

    /**
     * Show the form for editing the specified resource.
     */
    public function edit(Team $team)
    {
        if(!auth()->user()->can('teams.edit') || !auth()->user()->can('teams.manage.'.$team->id)) {
            return redirect()->back()->with('error', 'Du hast nicht die benötigte Berechtigung!');
        }

        //
        return view('gamemanage.edit', ['team' => $team, 'users' => User::all(), 'games' => auth()->user()->games]);
    }

    /**
     * Update the specified resource in storage.
     */
    public function update(Request $request, Team $team)
    {
        if(!auth()->user()->can('teams.edit') || !auth()->user()->can('teams.manage.'.$team->id)) {
            return redirect()->back()->with('error', 'Du hast nicht die benötigte Berechtigung!');
        }

        //
        $validator = Validator::make($request->only(['name', 'short', 'leader', 'game']), [
            'name' => ['required', 'string', 'max:100', Rule::unique(Team::class)->ignore($team->id)],
            'short' => ['required', 'string', 'max:20', Rule::unique(Team::class)->ignore($team->id)],
            'leader' => ['required', Rule::exists('users', 'id')],
            'game' => ['required', Rule::exists('games', 'id')],
        ]);

        if($validator->fails()) {
            return redirect()->back()->withInput()->withErrors($validator->errors());
        }

        $input = $validator->validated();
        $perm = Permission::firstOrCreate(['name' => 'teams.manage.'.$team->id], ['description' => 'Manage Team '.$team->name, 'only_user' => true]);
        $leader = User::find($input['leader']);

        $team->leader->revokePermissionTo($perm);

        $team->fill($request->only(['name', 'short']));
        $team->leader_id = $input['leader'];
        $team->game_id = $input['game'];

        if(!$team->save()) {
            return redirect()->back()->withInput()->with('error', 'Team konnte nicht gespeichert werden!');
        }

        $leader->givePermissionTo($perm);

        return redirect()->route('gamemanage.index')->with('success', 'Team wurde gespeichert.');
    }

    /**
     * Remove the specified resource from storage.
     */
    public function delete(Team $team)
    {
        if(!auth()->user()->can('teams.delete') || !auth()->user()->can('teams.manage.'.$team->id)) {
            return redirect()->back()->with('error', 'Du hast nicht die benötigte Berechtigung!');
        }

        return view('gamemanage.delete', ['team' => $team]);
    }

    /**
     * Remove the specified resource from storage.
     */
    public function destroy(Team $team)
    {
        if(!auth()->user()->can('teams.delete') || !auth()->user()->can('teams.manage.'.$team->id)) {
            return redirect()->back()->with('error', 'Du hast nicht die benötigte Berechtigung!');
        }

        if(!Permission::where('name', 'teams.manage.'.$team->id)->delete() || !$team->delete()) {
            return redirect()->route('gamemanage.index')->with('error', 'Team konnte nicht gelöscht werden.');
        }

        return redirect()->route('gamemanage.index')->with('success', 'Team wurde gelöscht.');
    }
}
